@@ -26,8 +26,6 @@
 #include "packageloader.h"
 #include "packagestructure.h"
 #include "private/package_p.h"
-// #include "private/packages_p.h"
-#include "private/packagejob_p.h"
 #include "private/packageloader_p.h"
 
 namespace KPackage
@@ -39,9 +37,7 @@ Package::Package(PackageStructure *structure)
 
     if (d->structure) {
         d->structure.data()->initPackage(this);
-        auto desc = i18n("Desktop file that describes this package.");
-        addFileDefinition("metadata", QStringLiteral("metadata.json"), desc);
-        addFileDefinition("metadata", QStringLiteral("metadata.desktop"), desc);
+        addFileDefinition("metadata", QStringLiteral("metadata.json"));
     }
 }
 
@@ -50,13 +46,7 @@ Package::Package(const Package &other)
 {
 }
 
-Package::~Package()
-{
-    // guard against deletion on application shutdown
-    if (PackageDeletionNotifier::self()) {
-        Q_EMIT PackageDeletionNotifier::self()->packageDeleted(this);
-    }
-}
+Package::~Package() = default;
 
 Package &Package::operator=(const Package &rhs)
 {
@@ -98,18 +88,10 @@ bool Package::isValid() const
     // search for the file in all prefixes and in all possible paths for each prefix
     // even if it's a big nested loop, usually there is one prefix and one location
     // so shouldn't cause too much disk access
-    QHashIterator<QByteArray, ContentStructure> it(d->contents);
-
-    while (it.hasNext()) {
-        it.next();
-        if (!it.value().required) {
-            continue;
-        }
-
-        const QString foundPath = filePath(it.key(), {});
-        if (foundPath.isEmpty()) {
-            // qCWarning(KPACKAGE_LOG) << "Could not find required" << (it.value().directory ? "directory" : "file") << it.key() << "for package" << path() <<
-            // "should be" << it.value().paths;
+    for (auto it = d->contents.cbegin(), end = d->contents.cend(); it != end; ++it) {
+        if (it.value().required && filePath(it.key()).isEmpty()) {
+            qCWarning(KPACKAGE_LOG) << "Could not find required" << (it.value().directory ? "directory" : "file") << it.key() << "for package" << path()
+                                    << "should be" << it.value().paths;
             d->valid = false;
             break;
         }
@@ -118,21 +100,9 @@ bool Package::isValid() const
     return d->valid;
 }
 
-#if KPACKAGE_BUILD_DEPRECATED_SINCE(5, 106)
-QString Package::name(const QByteArray &key) const
-{
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constFind(key);
-    if (it == d->contents.constEnd()) {
-        return QString();
-    }
-
-    return it.value().name;
-}
-#endif
-
 bool Package::isRequired(const QByteArray &key) const
 {
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constFind(key);
+    auto it = d->contents.constFind(key);
     if (it == d->contents.constEnd()) {
         return false;
     }
@@ -142,7 +112,7 @@ bool Package::isRequired(const QByteArray &key) const
 
 QStringList Package::mimeTypes(const QByteArray &key) const
 {
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constFind(key);
+    auto it = d->contents.constFind(key);
     if (it == d->contents.constEnd()) {
         return QStringList();
     }
@@ -266,12 +236,12 @@ QString PackagePrivate::unpack(const QString &filePath)
         tempRoot = tempdir.path() + QLatin1Char('/');
         source->copyTo(tempRoot);
 
-        if (!QFile::exists(tempdir.path() + QLatin1String("/metadata.json")) && !QFile::exists(tempdir.path() + QLatin1String("/metadata.desktop"))) {
-            // search metadata.desktop, the zip file might have the package contents in a subdirectory
+        if (!QFile::exists(tempdir.path() + QLatin1String("/metadata.json"))) {
+            // search metadata.json, the zip file might have the package contents in a subdirectory
             QDir unpackedPath(tempdir.path());
             const auto entries = unpackedPath.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
             for (const auto &pack : entries) {
-                if (QFile::exists(pack.filePath() + QLatin1String("/metadata.json")) || QFile::exists(pack.filePath() + QLatin1String("/metadata.desktop"))) {
+                if (QFile::exists(pack.filePath() + QLatin1String("/metadata.json"))) {
                     tempRoot = pack.filePath() + QLatin1Char('/');
                 }
             }
@@ -415,13 +385,12 @@ QStringList Package::entryList(const QByteArray &key) const
         return QStringList();
     }
 
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constFind(key);
+    const auto it = d->contents.constFind(key);
     if (it == d->contents.constEnd()) {
-        // qCDebug(KPACKAGE_LOG) << "couldn't find" << key;
+        qCWarning(KPACKAGE_LOG) << "couldn't find" << key << "when trying to list entries";
         return QStringList();
     }
 
-    // qCDebug(KPACKAGE_LOG) << "going to list" << key;
     QStringList list;
     for (const QString &prefix : std::as_const(d->contentsPrefixPaths)) {
         // qCDebug(KPACKAGE_LOG) << "     looking in" << prefix;
@@ -431,7 +400,6 @@ QStringList Package::entryList(const QByteArray &key) const
             if (it.value().directory) {
                 // qCDebug(KPACKAGE_LOG) << "it's a directory, so trying out" << d->path + prefix + path;
                 QDir dir(d->path + prefix + path);
-
                 if (d->externalPaths) {
                     list += dir.entryList(QDir::Files | QDir::Readable);
                 } else {
@@ -544,15 +512,6 @@ void Package::setPath(const QString &path)
 
         Q_ASSERT(QFile::exists(dir.canonicalPath()));
 
-        // if it has a contents.rcc, give priority to it
-        if (dir.exists(QStringLiteral("contents.rcc"))) {
-            d->rccPath = dir.absoluteFilePath(QStringLiteral("contents.rcc"));
-            QResource::registerResource(d->rccPath);
-
-            // we need just the plugin name here, never the absolute path
-            dir = QDir(QStringLiteral(":/") + defaultPackageRoot() + QStringView(path).mid(path.lastIndexOf(QLatin1Char('/'))));
-        }
-
         d->path = dir.canonicalPath();
         // canonicalPath() does not include a trailing slash (unless it is the root dir)
         if (!d->path.endsWith(QLatin1Char('/'))) {
@@ -621,13 +580,6 @@ void Package::setContentsPrefixPaths(const QStringList &prefixPaths)
     }
 }
 
-#if KPACKAGE_BUILD_DEPRECATED_SINCE(5, 21)
-QString Package::contentsHash() const
-{
-    return QString::fromLocal8Bit(cryptographicHash(QCryptographicHash::Sha1));
-}
-#endif
-
 QByteArray Package::cryptographicHash(QCryptographicHash::Algorithm algorithm) const
 {
     if (!d->valid) {
@@ -636,9 +588,8 @@ QByteArray Package::cryptographicHash(QCryptographicHash::Algorithm algorithm) c
     }
 
     QCryptographicHash hash(algorithm);
-    const QString metadataPath = QFile::exists(d->path + QLatin1String("metadata.json"))
-        ? d->path + QLatin1String("metadata.json")
-        : QFile::exists(d->path + QLatin1String("metadata.desktop")) ? d->path + QLatin1String("metadata.desktop") : QString();
+    const QString guessedMetaDataJson = d->path + QLatin1String("metadata.json");
+    const QString metadataPath = QFile::exists(guessedMetaDataJson) ? guessedMetaDataJson : QString();
     if (!metadataPath.isEmpty()) {
         QFile f(metadataPath);
         if (f.open(QIODevice::ReadOnly)) {
@@ -666,13 +617,13 @@ QByteArray Package::cryptographicHash(QCryptographicHash::Algorithm algorithm) c
     return hash.result().toHex();
 }
 
-void Package::addDirectoryDefinition(const QByteArray &key, const QString &path, const QString &name)
+void Package::addDirectoryDefinition(const QByteArray &key, const QString &path)
 {
     const auto contentsIt = d->contents.constFind(key);
     ContentStructure s;
 
     if (contentsIt != d->contents.constEnd()) {
-        if (contentsIt->paths.contains(path) && contentsIt->directory == true && contentsIt->name == name) {
+        if (contentsIt->paths.contains(path) && contentsIt->directory == true) {
             return;
         }
         s = *contentsIt;
@@ -680,32 +631,25 @@ void Package::addDirectoryDefinition(const QByteArray &key, const QString &path,
 
     d.detach();
 
-    if (!name.isEmpty()) {
-        s.name = name;
-    }
-
     s.paths.append(path);
     s.directory = true;
 
     d->contents[key] = s;
 }
 
-void Package::addFileDefinition(const QByteArray &key, const QString &path, const QString &name)
+void Package::addFileDefinition(const QByteArray &key, const QString &path)
 {
     const auto contentsIt = d->contents.constFind(key);
     ContentStructure s;
 
     if (contentsIt != d->contents.constEnd()) {
-        if (contentsIt->paths.contains(path) && contentsIt->directory == true && contentsIt->name == name) {
+        if (contentsIt->paths.contains(path) && contentsIt->directory == true) {
             return;
         }
         s = *contentsIt;
     }
 
     d.detach();
-    if (!name.isEmpty()) {
-        s.name = name;
-    }
 
     s.paths.append(path);
     s.directory = false;
@@ -730,6 +674,7 @@ void Package::setRequired(const QByteArray &key, bool required)
 {
     QHash<QByteArray, ContentStructure>::iterator it = d->contents.find(key);
     if (it == d->contents.end()) {
+        qCWarning(KPACKAGE_LOG) << key << "is now a known key for the package. File is thus not set to being required";
         return;
     }
 
@@ -747,26 +692,21 @@ void Package::setDefaultMimeTypes(const QStringList &mimeTypes)
 
 void Package::setMimeTypes(const QByteArray &key, const QStringList &mimeTypes)
 {
-    QHash<QByteArray, ContentStructure>::iterator it = d->contents.find(key);
-    if (it == d->contents.end()) {
+    if (!d->contents.contains(key)) {
         return;
     }
 
     d.detach();
-    // have to find the item again after detaching: d->contents is a different object now
-    it = d->contents.find(key);
-    it.value().mimeTypes = mimeTypes;
+    d->contents[key].mimeTypes = mimeTypes;
 }
 
 QList<QByteArray> Package::directories() const
 {
     QList<QByteArray> dirs;
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constBegin();
-    while (it != d->contents.constEnd()) {
+    for (auto it = d->contents.cbegin(); it != d->contents.cend(); ++it) {
         if (it.value().directory) {
             dirs << it.key();
         }
-        ++it;
     }
     return dirs;
 }
@@ -774,12 +714,10 @@ QList<QByteArray> Package::directories() const
 QList<QByteArray> Package::requiredDirectories() const
 {
     QList<QByteArray> dirs;
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constBegin();
-    while (it != d->contents.constEnd()) {
+    for (auto it = d->contents.cbegin(); it != d->contents.cend(); ++it) {
         if (it.value().directory && it.value().required) {
             dirs << it.key();
         }
-        ++it;
     }
     return dirs;
 }
@@ -787,12 +725,10 @@ QList<QByteArray> Package::requiredDirectories() const
 QList<QByteArray> Package::files() const
 {
     QList<QByteArray> files;
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constBegin();
-    while (it != d->contents.constEnd()) {
+    for (auto it = d->contents.cbegin(); it != d->contents.cend(); ++it) {
         if (!it.value().directory) {
             files << it.key();
         }
-        ++it;
     }
     return files;
 }
@@ -800,77 +736,17 @@ QList<QByteArray> Package::files() const
 QList<QByteArray> Package::requiredFiles() const
 {
     QList<QByteArray> files;
-    QHash<QByteArray, ContentStructure>::const_iterator it = d->contents.constBegin();
-    while (it != d->contents.constEnd()) {
+    for (auto it = d->contents.cbegin(); it != d->contents.cend(); ++it) {
         if (!it.value().directory && it.value().required) {
             files << it.key();
         }
-        ++it;
     }
 
     return files;
 }
 
-KJob *Package::install(const QString &sourcePackage, const QString &packageRoot)
-{
-    if (!d->structure) {
-        return nullptr;
-    }
-
-    const QString src = sourcePackage;
-    setPath(src);
-    QString dest = packageRoot.isEmpty() ? defaultPackageRoot() : packageRoot;
-    KPackage::PackageLoader::self()->d->maxCacheAge = -1;
-
-    // use absolute paths if passed, otherwise go under share
-    if (!QDir::isAbsolutePath(dest)) {
-        dest = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + dest;
-    }
-
-    // qCDebug(KPACKAGE_LOG) << "Source: " << src;
-    // qCDebug(KPACKAGE_LOG) << "PackageRoot: " << dest;
-    KJob *j = d->structure.data()->install(this, src, dest);
-    return j;
-}
-
-KJob *Package::update(const QString &sourcePackage, const QString &packageRoot)
-{
-    if (!d->structure) {
-        return nullptr;
-    }
-
-    const QString src = sourcePackage;
-    setPath(src);
-    QString dest = packageRoot.isEmpty() ? defaultPackageRoot() : packageRoot;
-    KPackage::PackageLoader::self()->d->maxCacheAge = -1;
-
-    // use absolute paths if passed, otherwise go under share
-    if (!QDir::isAbsolutePath(dest)) {
-        dest = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + dest;
-    }
-
-    // qCDebug(KPACKAGE_LOG) << "Source: " << src;
-    // qCDebug(KPACKAGE_LOG) << "PackageRoot: " << dest;
-    KJob *j = d->structure.data()->update(this, src, dest);
-    return j;
-}
-
-KJob *Package::uninstall(const QString &packageName, const QString &packageRoot)
-{
-    KPackage::PackageLoader::self()->d->maxCacheAge = -1;
-    d->createPackageMetadata(packageRoot + QLatin1Char('/') + packageName);
-    if (!d->structure) {
-        return nullptr;
-    }
-    return d->structure.data()->uninstall(this, packageRoot);
-}
-
 PackagePrivate::PackagePrivate()
     : QSharedData()
-    , fallbackPackage(nullptr)
-    , externalPaths(false)
-    , valid(false)
-    , checkedValid(false)
 {
     contentsPrefixPaths << QStringLiteral("contents/");
 }
@@ -886,13 +762,6 @@ PackagePrivate::PackagePrivate(const PackagePrivate &other)
 
 PackagePrivate::~PackagePrivate()
 {
-    if (!rccPath.isEmpty()) {
-        // qresource register/unregisterpath is refcounted if we call it two times
-        // on the same path, the resource will actually be unregistered only when
-        // unregister is called 2 times
-        QResource::unregisterResource(rccPath);
-    }
-
     if (!tempRoot.isEmpty()) {
         QDir dir(tempRoot);
         dir.removeRecursively();
@@ -980,24 +849,14 @@ void PackagePrivate::updateHash(const QString &basePath, const QString &subPath,
 
 void PackagePrivate::createPackageMetadata(const QString &path)
 {
-    const bool isDir = QFileInfo(path).isDir();
-
-    if (isDir && QFile::exists(path + QStringLiteral("/metadata.json"))) {
-        metadata = KPluginMetaData::fromJsonFile(path + QStringLiteral("/metadata.json"));
-#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 92)
-    } else if (isDir && QFile::exists(path + QStringLiteral("/metadata.desktop"))) {
-        metadata = KPluginMetaData::fromDesktopFile(path + QStringLiteral("/metadata.desktop"), {QStringLiteral(":/kservicetypes5/kpackage-generic.desktop")});
-#endif
-    } else {
-        if (isDir) {
-            qCDebug(KPACKAGE_LOG) << "No metadata file in the package, expected it at:" << path;
-#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 92)
-        } else if (path.endsWith(QLatin1String(".desktop"))) {
-            metadata = KPluginMetaData::fromDesktopFile(path, {QStringLiteral(":/kservicetypes5/kpackage-generic.desktop")});
-#endif
+    if (QFileInfo(path).isDir()) {
+        if (const QString jsonPath = path + QLatin1String("/metadata.json"); QFileInfo::exists(jsonPath)) {
+            metadata = KPluginMetaData::fromJsonFile(jsonPath);
         } else {
-            metadata = KPluginMetaData::fromJsonFile(path);
+            qCDebug(KPACKAGE_LOG) << "No metadata file in the package, expected it at:" << jsonPath;
         }
+    } else {
+        metadata = KPluginMetaData::fromJsonFile(path);
     }
 }
 
@@ -1036,12 +895,4 @@ bool PackagePrivate::hasCycle(const KPackage::Package &package)
     return false;
 }
 
-Q_GLOBAL_STATIC(PackageDeletionNotifier, s_packageDeletionNotifier)
-PackageDeletionNotifier *PackageDeletionNotifier::self()
-{
-    return s_packageDeletionNotifier;
-}
-
 } // Namespace
-
-#include "private/moc_package_p.cpp"
