@@ -16,7 +16,6 @@
 #include <KTar>
 #include <kzip.h>
 
-
 #include "kpackage_debug.h"
 #include <QDir>
 #include <QFile>
@@ -75,22 +74,37 @@ bool removeFolder(QString folderPath)
     return folder.removeRecursively();
 }
 
-Q_GLOBAL_STATIC_WITH_ARGS(QStringList, metaDataFiles, (QStringList(QLatin1String("metadata.desktop")) << QLatin1String("metadata.json")))
-
-
 class PackageJobThreadPrivate
 {
 public:
     QString installPath;
     QString errorMessage;
+    std::function<void()> run;
     int errorCode;
 };
 
-PackageJobThread::PackageJobThread(QObject *parent)
-    : QThread(parent)
+PackageJobThread::PackageJobThread(PackageJob::OperationType type, const QString &src, const QString &dest, const QString &packagePath)
+    : QObject()
+    , QRunnable()
 {
     d = new PackageJobThreadPrivate;
     d->errorCode = KJob::NoError;
+    if (type == PackageJob::Install) {
+        d->run = [this, src, dest]() {
+            install(src, dest);
+        };
+    } else if (type == PackageJob::Update) {
+        d->run = [this, src, dest]() {
+            update(src, dest);
+        };
+    } else if (type == PackageJob::Uninstall) {
+        d->run = [this, packagePath]() {
+            uninstall(packagePath);
+        };
+
+    } else {
+        Q_UNREACHABLE();
+    }
 }
 
 PackageJobThread::~PackageJobThread()
@@ -98,21 +112,29 @@ PackageJobThread::~PackageJobThread()
     delete d;
 }
 
+void PackageJobThread::run()
+{
+    Q_ASSERT(d->run);
+    d->run();
+}
 bool PackageJobThread::install(const QString &src, const QString &dest)
 {
-    bool ok = installPackage(src, dest, Install);
+    bool ok = installPackage(src, dest, PackageJob::Install);
     Q_EMIT installPathChanged(d->installPath);
-    Q_EMIT jobThreadFinished(ok, d->errorMessage);
+    Q_EMIT jobThreadFinished(ok, errorCode(), d->errorMessage);
     return ok;
 }
 
 static QString resolveHandler(const QString &scheme)
 {
-    QString candidatePath = QStringLiteral(KDE_INSTALL_FULL_LIBEXECDIR_KF5 "/kpackagehandlers/%1handler").arg(scheme);
-    if (qEnvironmentVariableIsSet("KPACKAGE_DEP_RESOLVERS_PATH")) {
-        candidatePath = QStringLiteral("%1/%2handler").arg(QString::fromUtf8(qgetenv("KPACKAGE_DEP_RESOLVERS_PATH")), scheme);
+    QString envOverride = qEnvironmentVariable("KPACKAGE_DEP_RESOLVERS_PATH");
+    QStringList searchDirs;
+    if (!envOverride.isEmpty()) {
+        searchDirs.push_back(envOverride);
     }
-    return QFile::exists(candidatePath) ? candidatePath : QString();
+    searchDirs.append(QStringLiteral(KDE_INSTALL_FULL_LIBEXECDIR_KF "/kpackagehandlers"));
+    // We have to use QStandardPaths::findExecutable here to handle the .exe suffix on Windows.
+    return QStandardPaths::findExecutable(scheme + QLatin1String("handler"), searchDirs);
 }
 
 bool PackageJobThread::installDependency(const QUrl &destUrl)
@@ -132,14 +154,14 @@ bool PackageJobThread::installDependency(const QUrl &destUrl)
     return process.exitCode() == 0;
 }
 
-bool PackageJobThread::installPackage(const QString &src, const QString &dest, OperationType operation)
+bool PackageJobThread::installPackage(const QString &src, const QString &dest, PackageJob::OperationType operation)
 {
     QDir root(dest);
     if (!root.exists()) {
         QDir().mkpath(dest);
         if (!root.exists()) {
             d->errorMessage = i18n("Could not create package root directory: %1", dest);
-            d->errorCode = Package::JobError::RootCreationError;
+            d->errorCode = PackageJob::JobError::RootCreationError;
             // qCWarning(KPACKAGE_LOG) << "Could not create package root directory: " << dest;
             return false;
         }
@@ -148,7 +170,7 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
     QFileInfo fileInfo(src);
     if (!fileInfo.exists()) {
         d->errorMessage = i18n("No such file: %1", src);
-        d->errorCode = Package::JobError::PackageFileNotFoundError;
+        d->errorCode = PackageJob::JobError::PackageFileNotFoundError;
         return false;
     }
 
@@ -178,7 +200,7 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
         } else {
             // qCWarning(KPACKAGE_LOG) << "Could not open package file, unsupported archive format:" << src << mimetype.name();
             d->errorMessage = i18n("Could not open package file, unsupported archive format: %1 %2", src, mimetype.name());
-            d->errorCode = Package::JobError::UnsupportedArchiveFormatError;
+            d->errorCode = PackageJob::JobError::UnsupportedArchiveFormatError;
             return false;
         }
 
@@ -186,7 +208,7 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
             // qCWarning(KPACKAGE_LOG) << "Could not open package file:" << src;
             delete archive;
             d->errorMessage = i18n("Could not open package file: %1", src);
-            d->errorCode = Package::JobError::PackageOpenError;
+            d->errorCode = PackageJob::JobError::PackageOpenError;
             return false;
         }
 
@@ -210,38 +232,31 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
     }
 
     QDir packageDir(path);
-    QFileInfoList entries = packageDir.entryInfoList(*metaDataFiles);
+    QFileInfoList entries = packageDir.entryInfoList(QStringList(QLatin1String("metadata.json")));
     KPluginMetaData meta;
     if (!entries.isEmpty()) {
         const QString metadataFilePath = entries.first().filePath();
-        if (metadataFilePath.endsWith(QLatin1String(".desktop"))) {
-            meta = KPluginMetaData::fromDesktopFile(metadataFilePath, {QStringLiteral(":/kservicetypes5/kpackage-generic.desktop")});
-        } else {
-            QFile f(metadataFilePath);
-            if (!f.open(QIODevice::ReadOnly)) {
-                qCWarning(KPACKAGE_LOG) << "Couldn't open metadata file" << src << path;
-                d->errorMessage = i18n("Could not open metadata file: %1", src);
-                d->errorCode = Package::JobError::MetadataFileMissingError;
-                return false;
-            }
-            QJsonObject metadataObject = QJsonDocument::fromJson(f.readAll()).object();
-            meta = KPluginMetaData(metadataObject, QString(), metadataFilePath);
-        }
+        meta = KPluginMetaData::fromJsonFile(metadataFilePath);
+    } else {
+        qCWarning(KPACKAGE_LOG) << "Couldn't open metadata file" << src << path;
+        d->errorMessage = i18n("Could not open metadata file: %1", src);
+        d->errorCode = PackageJob::JobError::MetadataFileMissingError;
+        return false;
     }
 
     if (!meta.isValid()) {
         qCDebug(KPACKAGE_LOG) << "No metadata file in package" << src << path;
         d->errorMessage = i18n("No metadata file in package: %1", src);
-        d->errorCode = Package::JobError::MetadataFileMissingError;
+        d->errorCode = PackageJob::JobError::MetadataFileMissingError;
         return false;
     }
 
     QString pluginName = meta.pluginId();
     qCDebug(KPACKAGE_LOG) << "pluginname: " << meta.pluginId();
-    if (pluginName.isEmpty()) {
-        // qCWarning(KPACKAGE_LOG) << "Package plugin name not specified";
-        d->errorMessage = i18n("Package plugin name not specified: %1", src);
-        d->errorCode = Package::JobError::PluginNameMissingError;
+    if (pluginName == QLatin1String("metadata")) {
+        // qCWarning(KPACKAGE_LOG) << "Package plugin id not specified";
+        d->errorMessage = i18n("Package plugin id not specified: %1", src);
+        d->errorCode = PackageJob::JobError::PluginIdInvalidError;
         return false;
     }
 
@@ -249,9 +264,9 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
     // bad characters into the paths used for removal.
     const QRegularExpression validatePluginName(QStringLiteral("^[\\w\\-\\.]+$")); // Only allow letters, numbers, underscore and period.
     if (!validatePluginName.match(pluginName).hasMatch()) {
-        // qCDebug(KPACKAGE_LOG) << "Package plugin name " << pluginName << "contains invalid characters";
-        d->errorMessage = i18n("Package plugin name %1 contains invalid characters", pluginName);
-        d->errorCode = Package::JobError::PluginNameInvalidError;
+        // qCDebug(KPACKAGE_LOG) << "Package plugin id " << pluginName << "contains invalid characters";
+        d->errorMessage = i18n("Package plugin id %1 contains invalid characters", pluginName);
+        d->errorCode = PackageJob::JobError::PluginIdInvalidError;
         return false;
     }
 
@@ -262,25 +277,28 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
     targetName.append(pluginName);
 
     if (QFile::exists(targetName)) {
-        if (operation == Update) {
-            KPluginMetaData oldMeta(targetName + QLatin1String("/metadata.desktop"));
+        if (operation == PackageJob::Update) {
+            KPluginMetaData oldMeta;
+            if (QString jsonPath = targetName + QLatin1String("/metadata.json"); QFileInfo::exists(jsonPath)) {
+                oldMeta = KPluginMetaData::fromJsonFile(jsonPath);
+            }
 
-            if (readKPackageTypes(oldMeta) != readKPackageTypes(meta)) {
+            if (readKPackageType(oldMeta) != readKPackageType(meta)) {
                 d->errorMessage = i18n("The new package has a different type from the old version already installed.");
-                d->errorCode = Package::JobError::UpdatePackageTypeMismatchError;
+                d->errorCode = PackageJob::JobError::UpdatePackageTypeMismatchError;
             } else if (isVersionNewer(oldMeta.version(), meta.version())) {
                 const bool ok = uninstallPackage(targetName);
                 if (!ok) {
                     d->errorMessage = i18n("Impossible to remove the old installation of %1 located at %2. error: %3", pluginName, targetName, d->errorMessage);
-                    d->errorCode = Package::JobError::OldVersionRemovalError;
+                    d->errorCode = PackageJob::JobError::OldVersionRemovalError;
                 }
             } else {
                 d->errorMessage = i18n("Not installing version %1 of %2. Version %3 already installed.", meta.version(), meta.pluginId(), oldMeta.version());
-                d->errorCode = Package::JobError::NewerVersionAlreadyInstalledError;
+                d->errorCode = PackageJob::JobError::NewerVersionAlreadyInstalledError;
             }
         } else {
             d->errorMessage = i18n("%1 already exists", targetName);
-            d->errorCode = Package::JobError::PackageAlreadyInstalledError;
+            d->errorCode = PackageJob::JobError::PackageAlreadyInstalledError;
         }
 
         if (d->errorCode != KJob::NoError) {
@@ -290,12 +308,18 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
     }
 
     // install dependencies
-    const QStringList dependencies = KPluginMetaData::readStringList(meta.rawData(), QStringLiteral("X-KPackage-Dependencies"));
+    const QStringList optionalDependencies{QStringLiteral("sddmtheme.knsrc")};
+    const QStringList dependencies = meta.value(QStringLiteral("X-KPackage-Dependencies"), QStringList());
     for (const QString &dep : dependencies) {
         QUrl depUrl(dep);
+        const QString knsrcFilePath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("knsrcfiles/") + depUrl.host());
+        if (knsrcFilePath.isEmpty() && optionalDependencies.contains(depUrl.host())) {
+            qWarning() << "Skipping depdendency due to knsrc files being missing" << depUrl;
+            continue;
+        }
         if (!installDependency(depUrl)) {
             d->errorMessage = i18n("Could not install dependency: '%1'", dep);
-            d->errorCode = Package::JobError::PackageCopyError;
+            d->errorCode = PackageJob::JobError::PackageCopyError;
             return false;
         }
     }
@@ -307,7 +331,7 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
         if (!ok) {
             // qCWarning(KPACKAGE_LOG) << "Could not move package to destination:" << targetName;
             d->errorMessage = i18n("Could not move package to destination: %1", targetName);
-            d->errorCode = Package::JobError::PackageMoveError;
+            d->errorCode = PackageJob::JobError::PackageMoveError;
             return false;
         }
     } else {
@@ -317,7 +341,7 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
         if (!ok) {
             // qCWarning(KPACKAGE_LOG) << "Could not copy package to destination:" << targetName;
             d->errorMessage = i18n("Could not copy package to destination: %1", targetName);
-            d->errorCode = Package::JobError::PackageCopyError;
+            d->errorCode = PackageJob::JobError::PackageCopyError;
             return false;
         }
     }
@@ -333,19 +357,18 @@ bool PackageJobThread::installPackage(const QString &src, const QString &dest, O
 
 bool PackageJobThread::update(const QString &src, const QString &dest)
 {
-    bool ok = installPackage(src, dest, Update);
+    bool ok = installPackage(src, dest, PackageJob::Update);
     Q_EMIT installPathChanged(d->installPath);
-    Q_EMIT jobThreadFinished(ok, d->errorMessage);
+    Q_EMIT jobThreadFinished(ok, errorCode(), d->errorMessage);
     return ok;
 }
 
 bool PackageJobThread::uninstall(const QString &packagePath)
 {
     bool ok = uninstallPackage(packagePath);
-    // qCDebug(KPACKAGE_LOG) << "emit installPathChanged " << d->installPath;
-    Q_EMIT installPathChanged(QString());
+    // Do not emit the install path changed, information about the removed package might be useful for consumers
     // qCDebug(KPACKAGE_LOG) << "Thread: installFinished" << ok;
-    Q_EMIT jobThreadFinished(ok, d->errorMessage);
+    Q_EMIT jobThreadFinished(ok, errorCode(), d->errorMessage);
     return ok;
 }
 
@@ -353,7 +376,7 @@ bool PackageJobThread::uninstallPackage(const QString &packagePath)
 {
     if (!QFile::exists(packagePath)) {
         d->errorMessage = packagePath.isEmpty() ? i18n("package path was deleted manually") : i18n("%1 does not exist", packagePath);
-        d->errorCode = Package::JobError::PackageFileNotFoundError;
+        d->errorCode = PackageJob::JobError::PackageFileNotFoundError;
         return false;
     }
     QString pkg;
@@ -373,16 +396,16 @@ bool PackageJobThread::uninstallPackage(const QString &packagePath)
     bool ok = removeFolder(packagePath);
     if (!ok) {
         d->errorMessage = i18n("Could not delete package from: %1", packagePath);
-        d->errorCode = Package::JobError::PackageUninstallError;
+        d->errorCode = PackageJob::JobError::PackageUninstallError;
         return false;
     }
 
     return true;
 }
 
-Package::JobError PackageJobThread::errorCode() const
+PackageJob::JobError PackageJobThread::errorCode() const
 {
-    return static_cast<Package::JobError>(d->errorCode);
+    return static_cast<PackageJob::JobError>(d->errorCode);
 }
 
 } // namespace KPackage
